.ig XC
%  nroff -man table.man | more
%n nroff -man table.man | lpr
%t ptroff -man table.man -Plw
.XC
.TH TABLE 3 "Feb 22th 1990"
.SH NAME 
table \- table handling routines for role playing and computer games
.SH SYNOPSIS
.nf
#include "table.h"

int tb_read(filename,table) char *filename ; table_t **table ; 

int tb_use(table,...,result) table_t *table ; ... char **result ;

int tb_print(table,filename,way) table_t *table ; char *filename ; int way ;

int tb_many(table,num,...,result) table_t *table ; int num ; ... char **result ; 
int tb_free(table) table_t *table ; 

int tb_query(table,query,result) table_t *table ; int query ; int *result ;
.fi
.SH DESCRIPTION
.IP tb_read
Given a pointer to a pointer to a table, and the name of a file, this 
routine reads the text representation of a table into the table variable,
using malloc(3) to allocate memory as needed.
.IP tb_use
Given a poniter to a table, a pointer to a pointer to characters, and
(optionally) other arguments, this routine uses the table and a random
number generator to "roll" on the table.  The number and type of optional
arguments depends on the type of table being used.  Simple tables
require no extra arguments, but multi and change tables require a 
string which determines which column on the table to use.  For example:

.nf
table_t *foo ;
char *result ;
tb_use(foo,&result) ;           /* assumes foo is a simple table */
tb_use(foo,"column1",&result) ; /* assumes foo is a multi table */
tb_use(foo,"column1",&result) ; /* assumes foo is a change table */
.fi

Note that the result points at a static buffer, which must not be changed
and which may disappear during the next tb_use call.  
.IP tb_print
Given a pointer to a table and a file name, this routine prints out a
copy of the file.  The third argument determines the form the table is
printed out in.  Right now, 1 prints it out in troff/tbl format, and
0 is reserved for future work.  This routine is useful for debugging, as
well as to generate documentation.
.IP tb_many
This routine is similar to tb_use, execpt that it returns several rolls
from the table which are different.  The extra argument ``num'' is the
number of rolls.  The result should be an array of character pointers
large enough to hold all the answers.  
.IP tb_free
Given a pointer to a table, this routines frees the space associated 
with the table.  After calling this routine the only function call which
is legal on the table is tb_read.  This routine calls free(3).
.IP tb_query
Given a pointer to a table, and a question about that table, result will
be filled with the answer to the question.  Currently supported queries 
include:
.RS
.IP TB_Q_MAX
Returns the largest roll which can be made on this table.
.IP TB_Q_MIN
Returns the smallest roll which can be made on this table.
.IP TB_Q_AVE
Returns the average roll made on this table.
.RE
.bp
.SH "TABLE TYPES"
.PP
This package support several different types of tables, and new types 
can be added with a recompile (see the man page on table-add).  Here
is a description of all of the table types which are currently supported.
.IP "Simple"
This is the simplest form of table.  It is a list of rolls and the results
for each roll.  An example would be a table of possible races for NPCs, or
a list of possible home towns.
.IP "Multiple"
These tables contain one column for the dice roll, but several columns 
each of which contains a result in different situations.  An example
might be a PC hight table, with different columns for each race.  Or a
yearly income table, with different columns for each class.  When using
these tables, you must specify which column is important.
.IP "SimpleBig"
This table is very similar to the simple table.  The difference is in
formatting.  This table is designed to return large chuncks of text,
like sentences and paragraphs.  The simple tables are designed to return
words or lines.
.bp
.SH "FILE FORMATS"
.PP
The files read by tb_read must be formatted correctly.  The format for
each type of table (simple, multi, etc.) is different, but the first
line is always the same:  the name of the table followed by its type.
.PP
Simple tables have a line with the type of roll to make and the number
of rows in the table, seperated by a tab.  The third line is the source
of the table.  The fourth, and all following lines, are rows of the table,
each row contains a number (or two numbers seperated by a dash) followed
by a tab, and the result.
.PP
Multiple tables are more complicated.  Their second row is the roll, the
number of rows, and the number of columns; all separated by tabs.  The number
of columns does not include the results column.  The third line is the source
of the table.  The fourth line are the column headers; each header is 
separated by a tab.  The first column is used for the roll, so it is 
untitled.  The fifth line, and all other rows are rows of the table; they
start with the roll and then have each result.  Each field is separated by
a tab.
.bp
.SH "EXAMPLE FILES"
.LP
Here is a sample simple table file (The `::' charaters are tabs):
.nf
Pay-off results table::Simple
2d6::9
p13 of The Complete Tavern
2::Banker loses triple, passes dice
3::Banker loses double, passes dice
4::Banker loses double
5::Banker loses
6::Banker loses half of bet
7::Stand off, rethrow
8-9::Banker wins bet
10-11::Banker double bet
12::Banker triple bet
.fi
.LP
Here is an example multiple table file ("::" are tabs):
.nf
Basic Test::Multiple
2d6::6::4	
my mind
-::Elf::Human::Dwarf::Troll	
2::e2::h2::d2::t2
3::e3::h3::d3::t3
4-7::e4::h4::d4::t4
8-9::e8::h8::d8::t8
10::e10::h10::d10::t10
11-12::e11::h11::d11::t11
.fi
.bp
.SH "EXAMPLE CODE"
.nf
/*
 * Routine to tralate a table stored in file infile into a troff/tbl
 * document, which is in outfile.
 */
tb2tbl(infile,outfile) char *infile ; char *outfile ;
{
    table_t *tab ;
    int sts ;

    sts = tb_read(infile,&tab) ;  assert(sts==TB_E_OK) ;
    sts = tb_print(tab,outfile,1) ;  assert(sts==TB_E_OK) ;
    sts = tb_free(tab) ;          assert(sts==TB_E_OK) ;
}
/*
 * Routine read in a table, and then rolls on it repeatedly, printing
 * the result of each roll to the screen.
 */

roll(tabfile,times) char *tabfile ; int times ;
{
    table_t *tab ;	
    char *str ;
    int sts ;
    int i ;

    sts = tb_read(infile,&tab)  ;  assert(sts==TB_E_OK) ;
    for(i=0 ; i<times ; i++) {
        sts = tb_use(tab,&str) ;
	printf("result %d was %s\\n",i+1,str) ;
    }
    sts = tb_free(tab) ;          assert(sts==TB_E_OK) ;
}
.fi
.bp
.SH DIAGNOSTICS
.TP 15
TB_E_OK
Everything worked fine.
.TP 15
TB_E_MISC	
General purpose error message.  Something went wrong.
.TP 15
TB_E_NOTYET
Feature not yet implemented.  
.TP 15
TB_E_NOFILE
The external table file you wanted to read does not exist.
.TP 15
TB_E_NO_COL
No column in the table iwth that name.  For multiple column tables, you
must pass a column name to tb_use or tb_choose, but your name does not
match one in the table.
.TP 15
TB_E_NOTYPE
The type of the table you passed to a tb_ routine is not a legal type.
Currently the legal types are 1 (Simple) and 2 (Multi).  Either you are
experimenting with new types, or (more likely) the table pointer you
passed to a tb_routine points to garbage.
.TP 15
TB_E_TOOMANY
You have asked the tb_many routine for too many answers.  The the table 
only contains 10 possible answers (for example) and you ask for 12,
you will get this error.  Since tb_many must return as many unique answers
as you ask for, it will return this error if there are not that many 
unique answers in the table.
.SH "SEE ALSO"
.PP
These routines use ra_random(3) to generate random numbers.
.SH "BUGS"
.IP \(bu
Using strange file names to specify stdout and stderr in tb_print is
a sleezy hack.  Also, there is no way to write this to a file descriptor
which is already open.
.SH AUTHOR
.PP
The table routines were written by Joshua Levy at Atherton Technology.
.br
joshua@atherton.com   Home: (415) 968-3718   Work: (408) 734-9822
.IP "\(co 1989 Joshua Levy"
This man page and the code it describes is covered by the GNU general
public license, commonly called the "copyleft".  It can only be used or
redistributed under the terms of that agreement.  Persons or corporations
wishing to use this code in ways not allowed by the GNU license should 
contact the author to discuss other licensing agreements.
.br


