.ig XC
%  nroff -man tb-add.man | more
%n nroff -man tb-add.man | lpr
%t ptroff -man tb-add.man -Plw
.XC
.TH TABLE-ADD 3 "Feb 22th 1990"
.SH NAME 
table-add \- instructions for adding new table and messagte types to the table routines
.SH SYNOPSIS
.PP
This page describes how to add new table types to the table package.
It also discusses how to add new messages.
(It is not really tested, but should give you a flavor of what to do.)
.SH OVERVIEW
.PP
The code in table.c is a very simple dispatch mechanism for a very 
simple object oriented system.  To add a new table type you must
first tell the dispatch system about it, and then write the "message
handling" or method routines, which the dispatcher will call for you.
.PP
The most complicated of these routines is the read routine, which 
reads the table from a disk file.  This routine implicitly defines
the text format used to store the new table type.  The format is
very important, since that is how tables are created and modified
by people.
.SH DESCRIPTION
.IP "Create a sample table file."
I always start with the format of the table file.  I
create a sample which will be used as the first test case.  The cat
command with the -e and -t flags is great for making sure you put
tabs in all the right price.  Here are the guidelines I use in designing
these files:
.RS
.IP 1
The first line should contain the table's title and its type.
.IP 2
The second line should contain the dice roll used on this tabel,
and all information about the size of the table.  After reading
the first two lines, the tb_read routine should have enough 
information to allocate space for the table structure.
.IP 3
The third line should contain the source of this table.  A title and
page number, or the person who came up with the table are fine here.
.IP 4
Other lines might contain other control type information.
.IP 5
Finally, the lines which describe the actual table.  Ideally, the
file should look like the table, so that it is as easy as possible
to copy tables from books and magazine articles.
.RE
.IP "Add a new type to the tba_type table."
About a third of the way down the table.c file is an array of structures
called tba_type.  Add a new structure to this array and the second to last
item.  The first field in the structure should be the (capitalized) name
of your new table type.  The second field should be it's unique number.
A sample new structure is "{"New",10}".  The new list would like like this:
{"Simple", 1}, {"Multiple",2}, {"New",3}, {NULL, 0}.  The number you used 
should be one more than the number currently in use.
.IP "Add new methods to the dispatch tables."
Right after the tba_type array are the arrays used to dispatch messages.
Add your routines after the routines already there.  If a given message
is legal, then put in illegal.  Note that table type 0 does not exist,
so an message sent to 0 is illegal.  That is why the first entry in each
array is illegal.
.IP "Create the new data structure."
This is pretty easy.  Make sure the first field is an integer called
type, so that the structure overlay code works correctly.  They just
translate each field from you table file to be a field in your structure.
Where possible, try to have similar fields in other table structures
overlay with fields in your structure.  This data structure, as well
as the declarations for your new method functions should be put in a
file called tablenew.h.  Add this file to the other .h files included
in table.h.  Your structure should be called tablenew_t.
.IP "Naming Convensions"
.PP
The file which contains your methods should be named tablenew.c, assuming
your file type is called new.  The any public functions which you write,
especially method functions, should be named tb_new_ followed by the name 
of the message (read, print, use, many, free).  
.IP "Write Methods"
.RS
.IP "read"
The first method I write is the read routine, since it is the most
difficult, and required for all the other routines. <<<add>>>
.IP
The interface for this program should look like this:
.br
int tb_new_read(rt,fp,title) table_t **rt ; FILE *fp ; char *title ; 
.br
.IP "tbl"
The second method I write is the tbl routine, since it tests the read
routine and makes sure that all the data really is put into the table
structure properly.
.IP
The interface for this program should look like this:
.br
int tb_new_tbl(tab,fp,way) table_t *tab ; FILE *fp ; int way ; 
.br
.IP "use"
.IP
The interface for this program should look like this:
.br
int tb_new_use(tab,result) table_t *tab ; char **result ;
.br
.IP "free"
.IP
The interface for this program should look like this:
.br
int tb_new_free(rt) table_t *rt ; 
.br
.RE
.IP "Modify the tb_use routine."
.IP "Add new table type to the test suite."
<<When there is a test suite for tables:>>
.SH "EXAMPLE CODE"
.PP
Currently there are two table types, simple and multi.  Both other these
can be used as sample code, to help you understand how all the pieces fit
together.
.SH DIAGNOSTICS
.PP
All tb_ functions return a negative integer to show that an error
has occured.  This error codes are listed in table.h, and should be
used by your code as much as possible.  If you need to add an new
error code follow these steps:
.IP 1
Add the code to table.h using the same format as the error codes already
there.
.IP 2
Modify the tb_error routine to print out a useful error message 
when the new error occures.
.SH "ADDING MESSAGES"
.PP
To add a new message to the table routines, you should follow the following
steps.  <example>
.IP "Add new handler routine."
.IP "Add new message array."
.IP "Add declarations to each table types's .h file"
.IP "Add message handlers to old table types."
.SH REPORTING
.PP
If you implement a new useful table type, please email it to me, so that
I can integrate it with future releases of the table subroutine package.
This is imporant, since otherwise, if I add a new type, and you add a
new type, they will likely have the same table type number, and cause
all sorts of problems.  The same is true of new error numbers.
.SH AUTHOR
.PP
The table routines were written by Joshua Levy at Atherton Technology.
.br
joshua@atherton.com   Home: (415) 968-3718   Work: (408) 734-9822
.IP "\(co 1989 Joshua Levy"
This man page and the code it describes is covered by the GNU general
public license, commonly called the "copyleft".  It can only be used or
redistributed under the terms of that agreement.  Persons or corporations
wishing to use this code in ways not allowed by the GNU license should 
contact the author to discuss other licensing agreements.
.br
